@@ -71,29 +71,6 @@ minetest.register_ore({
 	height_max     = -17,
 })
 
--- for float islands and far scaled
-minetest.register_ore({
-	ore_type       = "scatter",
-	ore            = "default:stone_with_coal",
-	wherein        = "default:stone",
-	clust_scarcity = 8*8*8,
-	clust_num_ores = 5,
-	clust_size     = 3,
-	height_min     = 200,
-	height_max     = 31000,
-})
-
-minetest.register_ore({
-	ore_type       = "scatter",
-	ore            = "default:stone_with_iron",
-	wherein        = "default:stone",
-	clust_scarcity = 9*9*9,
-	clust_num_ores = 5,
-	clust_size     = 3,
-	height_min     = 200,
-	height_max     = 31000,
-})
-
 minetest.register_on_generated(function(minp, maxp, seed)
 	-- Generate clay
 	if maxp.y >= 2 and minp.y <= 0 then
@@ -134,3 +111,16 @@ minetest.register_on_generated(function(minp, maxp, seed)
 	end
 end)
 
+--
+-- Register biome for biome API
+--
+
+minetest.register_biome({
+	name           = "Grassland",
+	-- Will use defaults of omitted parameters
+	y_min          = -31000,
+	y_max          = 31000,
+	heat_point     = 50,
+	humidity_point = 50,
+})
+
