@@ -31,6 +31,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "localplayer.h"
 #include "camera.h"
 #include "porting.h"
+#include "fontengine.h"
 #include <IGUIStaticText.h>
 
 #ifdef HAVE_TOUCHSCREENGUI
@@ -38,14 +39,11 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #endif
 
 Hud::Hud(video::IVideoDriver *driver, scene::ISceneManager* smgr,
-		gui::IGUIEnvironment* guienv, gui::IGUIFont *font,
-		u32 text_height, IGameDef *gamedef,
-		LocalPlayer *player, Inventory *inventory) {
+		gui::IGUIEnvironment* guienv, IGameDef *gamedef, LocalPlayer *player,
+		Inventory *inventory) {
 	this->driver      = driver;
 	this->smgr        = smgr;
 	this->guienv      = guienv;
-	this->font        = font;
-	this->text_height = text_height;
 	this->gamedef     = gamedef;
 	this->player      = player;
 	this->inventory   = inventory;
@@ -53,29 +51,29 @@ Hud::Hud(video::IVideoDriver *driver, scene::ISceneManager* smgr,
 	m_screensize       = v2u32(0, 0);
 	m_displaycenter    = v2s32(0, 0);
 	m_hotbar_imagesize = floor(HOTBAR_IMAGE_SIZE * porting::getDisplayDensity() + 0.5);
-	m_hotbar_imagesize *= g_settings->getFloat("gui_scaling");
+	m_hotbar_imagesize *= g_settings->getFloat("hud_scaling");
 	m_padding = m_hotbar_imagesize / 12;
 
 	const video::SColor hbar_color(255, 255, 255, 255);
 	for (unsigned int i=0; i < 4; i++ ){
 		hbar_colors[i] = hbar_color;
 	}
-	
+
 	tsrc = gamedef->getTextureSource();
-	
+
 	v3f crosshair_color = g_settings->getV3F("crosshair_color");
 	u32 cross_r = rangelim(myround(crosshair_color.X), 0, 255);
 	u32 cross_g = rangelim(myround(crosshair_color.Y), 0, 255);
 	u32 cross_b = rangelim(myround(crosshair_color.Z), 0, 255);
 	u32 cross_a = rangelim(g_settings->getS32("crosshair_alpha"), 0, 255);
 	crosshair_argb = video::SColor(cross_a, cross_r, cross_g, cross_b);
-	
+
 	v3f selectionbox_color = g_settings->getV3F("selectionbox_color");
 	u32 sbox_r = rangelim(myround(selectionbox_color.X), 0, 255);
 	u32 sbox_g = rangelim(myround(selectionbox_color.Y), 0, 255);
 	u32 sbox_b = rangelim(myround(selectionbox_color.Z), 0, 255);
 	selectionbox_argb = video::SColor(255, sbox_r, sbox_g, sbox_b);
-	
+
 	use_crosshair_image = tsrc->isKnownSourceImage("crosshair.png");
 
 	hotbar_image = "";
@@ -156,7 +154,7 @@ void Hud::drawItem(const ItemStack &item, const core::rect<s32>& rect, bool sele
 		video::SColor bgcolor2(128, 0, 0, 0);
 		if (!use_hotbar_image)
 			driver->draw2DRectangle(bgcolor2, rect, NULL);
-		drawItemStack(driver, font, item, rect, NULL, gamedef);
+		drawItemStack(driver, g_fontengine->getFont(), item, rect, NULL, gamedef);
 	}
 
 //NOTE: selectitem = 0 -> no selected; selectitem 1-based
@@ -240,11 +238,13 @@ void Hud::drawItems(v2s32 upperleftpos, s32 itemcount, s32 offset,
 
 
 void Hud::drawLuaElements(v3s16 camera_offset) {
+	u32 text_height = g_fontengine->getTextHeight();
+	irr::gui::IGUIFont* font = g_fontengine->getFont();
 	for (size_t i = 0; i != player->maxHudId(); i++) {
 		HudElement *e = player->getHud(i);
 		if (!e)
 			continue;
-		
+
 		v2s32 pos(floor(e->pos.X * (float) m_screensize.X + 0.5),
 				floor(e->pos.Y * (float) m_screensize.Y + 0.5));
 		switch (e->type) {
@@ -331,26 +331,22 @@ void Hud::drawStatbar(v2s32 pos, u16 corner, u16 drawdir, std::string texture,
 {
 	const video::SColor color(255, 255, 255, 255);
 	const video::SColor colors[] = {color, color, color, color};
-	
+
 	video::ITexture *stat_texture = tsrc->getTexture(texture);
 	if (!stat_texture)
 		return;
-		
+
 	core::dimension2di srcd(stat_texture->getOriginalSize());
 	core::dimension2di dstd;
 	if (size == v2s32()) {
 		dstd = srcd;
 	} else {
-		dstd.Height = size.Y * g_settings->getFloat("gui_scaling") *
-				porting::getDisplayDensity();
-		dstd.Width  = size.X * g_settings->getFloat("gui_scaling") *
-				porting::getDisplayDensity();
-
-		offset.X *= g_settings->getFloat("gui_scaling") *
-				porting::getDisplayDensity();
-
-		offset.Y *= g_settings->getFloat("gui_scaling") *
+		double size_factor = g_settings->getFloat("hud_scaling") *
 				porting::getDisplayDensity();
+		dstd.Height = size.Y * size_factor;
+		dstd.Width  = size.X * size_factor;
+		offset.X *= size_factor;
+		offset.Y *= size_factor;
 	}
 
 	v2s32 p = pos;
@@ -375,7 +371,7 @@ void Hud::drawStatbar(v2s32 pos, u16 corner, u16 drawdir, std::string texture,
 	}
 	steppos.X *= dstd.Width;
 	steppos.Y *= dstd.Height;
-	
+
 	for (s32 i = 0; i < count / 2; i++)
 	{
 		core::rect<s32> srcrect(0, 0, srcd.Width, srcd.Height);
@@ -385,7 +381,7 @@ void Hud::drawStatbar(v2s32 pos, u16 corner, u16 drawdir, std::string texture,
 		driver->draw2DImage(stat_texture, dstrect, srcrect, NULL, colors, true);
 		p += steppos;
 	}
-	
+
 	if (count % 2 == 1)
 	{
 		core::rect<s32> srcrect(0, 0, srcd.Width / 2, srcd.Height);
@@ -406,7 +402,7 @@ void Hud::drawHotbar(u16 playeritem) {
 		//silently ignore this we may not be initialized completely
 		return;
 	}
-	
+
 	s32 hotbar_itemcount = player->hud_hotbar_itemcount;
 	s32 width = hotbar_itemcount * (m_hotbar_imagesize + m_padding * 2);
 	v2s32 pos = centerlowerpos - v2s32(width / 2, m_hotbar_imagesize + m_padding * 3);
@@ -432,24 +428,26 @@ void Hud::drawHotbar(u16 playeritem) {
 	//////////////////////////// compatibility code to be removed //////////////
 	// this is ugly as hell but there's no other way to keep compatibility to
 	// old servers
-	if ( player->hud_flags & HUD_FLAG_HEALTHBAR_VISIBLE)
-		drawStatbar(v2s32(floor(0.5 * (float) m_screensize.X + 0.5),
-				floor(1 * (float) m_screensize.Y + 0.5)),
-				HUD_CORNER_UPPER, 0, "heart.png",
-				player->hp, v2s32((-10*24)-25,-(48+24+10)), v2s32(24,24));
+	if ((player->hud_flags & HUD_FLAG_HEALTHBAR_VISIBLE)) {
+		drawStatbar(v2s32(floor(0.5 * (float)m_screensize.X + 0.5),
+			floor(1 * (float) m_screensize.Y + 0.5)),
+			HUD_CORNER_UPPER, 0, "heart.png",
+			player->hp, v2s32((-10*24)-25,-(48+24+10)), v2s32(24,24));
+	}
 
 	if ((player->hud_flags & HUD_FLAG_BREATHBAR_VISIBLE) &&
-			(player->getBreath() < 11))
-		drawStatbar(v2s32(floor(0.5 * (float) m_screensize.X + 0.5),
-				floor(1 * (float) m_screensize.Y + 0.5)),
-				HUD_CORNER_UPPER, 0, "heart.png",
-				player->getBreath(), v2s32(25,-(48+24+10)), v2s32(24,24));
+			(player->getBreath() < 11)) {
+		drawStatbar(v2s32(floor(0.5 * (float)m_screensize.X + 0.5),
+			floor(1 * (float) m_screensize.Y + 0.5)),
+			HUD_CORNER_UPPER, 0, "bubble.png",
+			player->getBreath(), v2s32(25,-(48+24+10)), v2s32(24,24));
+	}
 	////////////////////////////////////////////////////////////////////////////
 }
 
 
 void Hud::drawCrosshair() {
-		
+
 	if (use_crosshair_image) {
 		video::ITexture *crosshair = tsrc->getTexture("crosshair.png");
 		v2u32 size  = crosshair->getOriginalSize();
@@ -479,7 +477,7 @@ void Hud::drawSelectionBoxes(std::vector<aabb3f> &hilightboxes) {
 void Hud::resizeHotbar() {
 	if (m_screensize != porting::getWindowSize()) {
 		m_hotbar_imagesize = floor(HOTBAR_IMAGE_SIZE * porting::getDisplayDensity() + 0.5);
-		m_hotbar_imagesize *= g_settings->getFloat("gui_scaling");
+		m_hotbar_imagesize *= g_settings->getFloat("hud_scaling");
 		m_padding = m_hotbar_imagesize / 12;
 		m_screensize = porting::getWindowSize();
 		m_displaycenter = v2s32(m_screensize.X/2,m_screensize.Y/2);
@@ -495,7 +493,7 @@ void drawItemStack(video::IVideoDriver *driver,
 {
 	if(item.empty())
 		return;
-	
+
 	const ItemDefinition &def = item.getDefinition(gamedef->idef());
 	video::ITexture *texture = gamedef->idef()->getInventoryTexture(def.name, gamedef);
 
