@@ -18,14 +18,20 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 */
 
 #include "player.h"
+
+#include <fstream>
+#include "util/numeric.h"
 #include "hud.h"
 #include "constants.h"
 #include "gamedef.h"
 #include "settings.h"
 #include "content_sao.h"
-#include "util/numeric.h"
+#include "filesys.h"
+#include "log.h"
+#include "porting.h"  // strlcpy
 
-Player::Player(IGameDef *gamedef):
+
+Player::Player(IGameDef *gamedef, const char *name):
 	touching_ground(false),
 	in_liquid(false),
 	in_liquid_stable(false),
@@ -33,7 +39,6 @@ Player::Player(IGameDef *gamedef):
 	is_climbing(false),
 	swimming_vertical(false),
 	camera_barely_in_ceiling(false),
-	light(0),
 	inventory(gamedef->idef()),
 	hp(PLAYER_MAX_HP),
 	hurt_tilt_timer(0),
@@ -42,26 +47,23 @@ Player::Player(IGameDef *gamedef):
 	keyPressed(0),
 // protected
 	m_gamedef(gamedef),
-	m_breath(-1),
+	m_breath(PLAYER_MAX_BREATH),
 	m_pitch(0),
 	m_yaw(0),
 	m_speed(0,0,0),
 	m_position(0,0,0),
 	m_collisionbox(-BS*0.30,0.0,-BS*0.30,BS*0.30,BS*1.75,BS*0.30),
-	m_last_pitch(0),
-	m_last_yaw(0),
-	m_last_pos(0,0,0),
-	m_last_hp(PLAYER_MAX_HP),
-	m_last_inventory(gamedef->idef())
+	m_dirty(false)
 {
-	updateName("<not set>");
+	strlcpy(m_name, name, PLAYERNAME_SIZE);
+
 	inventory.clear();
 	inventory.addList("main", PLAYER_INVENTORY_SIZE);
 	InventoryList *craft = inventory.addList("craft", 9);
 	craft->setWidth(3);
 	inventory.addList("craftpreview", 1);
 	inventory.addList("craftresult", 1);
-	m_last_inventory = inventory;
+	inventory.setModified(false);
 
 	// Can be redefined via Lua
 	inventory_formspec = "size[8,7.5]"
@@ -195,35 +197,28 @@ void Player::serialize(std::ostream &os)
 void Player::deSerialize(std::istream &is, std::string playername)
 {
 	Settings args;
-	
-	for(;;)
-	{
-		if(is.eof())
-			throw SerializationError
-					(("Player::deSerialize(): PlayerArgsEnd of player \"" + playername + "\" not found").c_str());
-		std::string line;
-		std::getline(is, line);
-		std::string trimmedline = trim(line);
-		if(trimmedline == "PlayerArgsEnd")
-			break;
-		args.parseConfigLine(line);
+
+	if (!args.parseConfigLines(is, "PlayerArgsEnd")) {
+		throw SerializationError("PlayerArgsEnd of player " +
+				playername + " not found!");
 	}
 
+	m_dirty = true;
 	//args.getS32("version"); // Version field value not used
 	std::string name = args.get("name");
-	updateName(name.c_str());
+	strlcpy(m_name, name.c_str(), PLAYERNAME_SIZE);
 	setPitch(args.getFloat("pitch"));
 	setYaw(args.getFloat("yaw"));
 	setPosition(args.getV3F("position"));
 	try{
 		hp = args.getS32("hp");
 	}catch(SettingNotFoundException &e) {
-		hp = 20;
+		hp = PLAYER_MAX_HP;
 	}
 	try{
 		m_breath = args.getS32("breath");
 	}catch(SettingNotFoundException &e) {
-		m_breath = 11;
+		m_breath = PLAYER_MAX_BREATH;
 	}
 
 	inventory.deSerialize(is);
@@ -241,9 +236,6 @@ void Player::deSerialize(std::istream &is, std::string playername)
 			inventory.getList("craftresult")->changeItem(0, ItemStack());
 		}
 	}
-
-	// Set m_last_*
-	checkModified();
 }
 
 u32 Player::addHud(HudElement *toadd)
@@ -294,7 +286,7 @@ void RemotePlayer::save(std::string savedir)
 	 */
 
 	// A player to deserialize files into to check their names
-	RemotePlayer testplayer(m_gamedef);
+	RemotePlayer testplayer(m_gamedef, "");
 
 	savedir += DIR_DELIM;
 	std::string path = savedir + m_name;
@@ -306,6 +298,7 @@ void RemotePlayer::save(std::string savedir)
 			if (!fs::safeWriteToFile(path, ss.str())) {
 				infostream << "Failed to write " << path << std::endl;
 			}
+			setModified(false);
 			return;
 		}
 		// Open file and deserialize
@@ -323,6 +316,7 @@ void RemotePlayer::save(std::string savedir)
 			if (!fs::safeWriteToFile(path, ss.str())) {
 				infostream << "Failed to write " << path << std::endl;
 			}
+			setModified(false);
 			return;
 		}
 		path = savedir + m_name + itos(i);
