@@ -80,9 +80,9 @@ public:
 		Modifying methods
 	*/
 
-	void addArea(VoxelArea &a)
+	void addArea(const VoxelArea &a)
 	{
-		if(getExtent() == v3s16(0,0,0))
+		if (hasEmptyExtent())
 		{
 			*this = a;
 			return;
@@ -94,9 +94,9 @@ public:
 		if(a.MaxEdge.Y > MaxEdge.Y) MaxEdge.Y = a.MaxEdge.Y;
 		if(a.MaxEdge.Z > MaxEdge.Z) MaxEdge.Z = a.MaxEdge.Z;
 	}
-	void addPoint(v3s16 p)
+	void addPoint(const v3s16 &p)
 	{
-		if(getExtent() == v3s16(0,0,0))
+		if(hasEmptyExtent())
 		{
 			MinEdge = p;
 			MaxEdge = p;
@@ -111,7 +111,7 @@ public:
 	}
 
 	// Pad with d nodes
-	void pad(v3s16 d)
+	void pad(const v3s16 &d)
 	{
 		MinEdge -= d;
 		MaxEdge += d;
@@ -137,6 +137,15 @@ public:
 	{
 		return MaxEdge - MinEdge + v3s16(1,1,1);
 	}
+
+	/* Because MaxEdge and MinEdge are included in the voxel area an empty extent
+	 * is not represented by (0, 0, 0), but instead (-1, -1, -1)
+	 */
+	bool hasEmptyExtent() const
+	{
+		return MaxEdge - MinEdge == v3s16(-1, -1, -1);
+	}
+
 	s32 getVolume() const
 	{
 		v3s16 e = getExtent();
@@ -146,7 +155,7 @@ public:
 	{
 		// No area contains an empty area
 		// NOTE: Algorithms depend on this, so do not change.
-		if(a.getExtent() == v3s16(0,0,0))
+		if(a.hasEmptyExtent())
 			return false;
 
 		return(
@@ -366,7 +375,8 @@ public:
 	*/
 	MapNode getNode(v3s16 p)
 	{
-		addArea(p);
+		VoxelArea voxel_area(p);
+		addArea(voxel_area);
 
 		if(m_flags[m_area.index(p)] & VOXELFLAG_NO_DATA)
 		{
@@ -383,7 +393,8 @@ public:
 	}
 	MapNode getNodeNoEx(v3s16 p)
 	{
-		addArea(p);
+		VoxelArea voxel_area(p);
+		addArea(voxel_area);
 
 		if(m_flags[m_area.index(p)] & VOXELFLAG_NO_DATA)
 		{
@@ -417,7 +428,8 @@ public:
 	}
 	MapNode & getNodeRef(v3s16 p)
 	{
-		addArea(p);
+		VoxelArea voxel_area(p);
+		addArea(voxel_area);
 		if(getFlagsRefUnsafe(p) & VOXELFLAG_NO_DATA)
 		{
 			/*dstream<<"EXCEPT: VoxelManipulator::getNode(): "
@@ -432,7 +444,8 @@ public:
 	}
 	void setNode(v3s16 p, const MapNode &n)
 	{
-		addArea(p);
+		VoxelArea voxel_area(p);
+		addArea(voxel_area);
 
 		m_data[m_area.index(p)] = n;
 		m_flags[m_area.index(p)] &= ~VOXELFLAG_NO_DATA;
@@ -499,7 +512,7 @@ public:
 	void print(std::ostream &o, INodeDefManager *nodemgr,
 			VoxelPrintMode mode=VOXELPRINT_MATERIAL);
 
-	void addArea(VoxelArea area);
+	void addArea(const VoxelArea &area);
 
 	/*
 		Copy data and set flags to 0
